package top.ninwoo.cloud;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.cloudcenter.CloudCenterMain;
import top.ninwoo.cloudcenter.service.CloudService;
import top.ninwoo.common.entity.*;

import java.util.*;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = CloudCenterMain.class)
public class CloudServiceImplTest {
    @Autowired
    private CloudService cloudService;

    @Autowired
    private RestTemplate restTemplate;

    @Before
    public void sendClusterConfigToEdgeNodeTest() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("127.0.0.1:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(2);
        DockerContainer container = new DockerContainer();
        container.setName("Run");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription containerDescription1 = new ContainerDescription();
        containerDescription1.setMode("normal");
        containerDescription1.setReplicas(3);
        DockerContainer container1 = new DockerContainer();
        container1.setName("APP");
        container1.setCommand("sh");
        container1.setImage("joliu/networktest");
        containerDescription1.setDockerContainer(container1);

        cds.add(containerDescription1);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"Run", "APP", "br:ovs1"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{{0,0,0},{0,0,0},{1,1,0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        // 下发配置到边缘服务器
        cloudService.sendClusterConfigToEdgeNode(clusterConfigs);

    }

    @After
    public void close() {
        // 清除集群
        cloudService.deleteClusterFromEdgeNode(11111l);
    }

    @Test
    public void deleteClusterFromEdgeNodeTest() {
    }

    @Test
    public void getIpListByAppNameFromRemoteIpService() {
        final String remoteIpService = "http://127.0.0.1:23333/getIpListByAppName?clusterId={clusterId}&appName={appName}";
        Map<String, Object> map = new HashMap<>();
        map.put("clusterId", 11111);
        map.put("appName", "test");
        ResponseEntity<HashSet> forEntity = restTemplate.getForEntity(remoteIpService, HashSet.class, map);
        Assert.assertTrue(forEntity.getStatusCode().is2xxSuccessful());
    }
;
    /**
     * 测试更新网络拓扑的功能模块
     */
    @Test
    public void updateTopologyTest() {
        NetworkTopology logicalTopology = new NetworkTopology();
        logicalTopology.setAppNames(new String[]{"Run", "APP"});
        logicalTopology.setTopology(new int[][]{{0,0},{0,0}});
        cloudService.updateLogicalTopology(11111l, logicalTopology);

        logicalTopology.setTopology(new int[][]{{0,0}, {1,0}});
        cloudService.updateLogicalTopology(11111l, logicalTopology);
    }
}
