package top.ninwoo.utils;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import top.ninwoo.common.entity.DockerContainer;
import top.ninwoo.utils.config.DockerConfig;
import top.ninwoo.utils.entity.ChainEntity;
import top.ninwoo.utils.entity.ChainType;
import top.ninwoo.utils.entity.TableType;
import top.ninwoo.utils.service.DockerService;
import top.ninwoo.utils.util.IptablesUtils;

import java.util.List;
import java.util.Map;

/**
 * @Author joliu
 * @Description
 * @Date　Create in 上午11:54 2019/10/29
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = DockerConfig.class)
public class IptablesUtilsTests {
    private static final Logger LOG = LoggerFactory.getLogger(IptablesUtilsTests.class);

    @Autowired
    IptablesUtils iptablesUtils;

    @Autowired
    DockerService dockerService;

    private DockerContainer dockerContainer;
    @Before
    public void init() {
        // 创建两个Docker
        dockerContainer = new DockerContainer();
        dockerContainer.setName("IptablesDocker");
        dockerContainer.setImage("joliu/networktest");
        dockerContainer.setCommand("sh");
        dockerContainer = dockerService.runDocker(dockerContainer);
    }

    @Test
    public void testAddIptables() {
        testShowIptablesDetail("initial");
        String s = iptablesUtils.addIptable(dockerContainer.getId(), "insert", ChainType.INPUT, "172.0.17.2", "", "DROP");
        iptablesUtils.addIptable(dockerContainer.getId(), "insert", ChainType.OUTPUT, "172.0.17.2", "", "DROP");
        iptablesUtils.addIptable(dockerContainer.getId(), "append", ChainType.INPUT, "172.0.17.3", "", "DROP");
        testShowIptablesDetail("下发三条条目");
        iptablesUtils.delIptable(dockerContainer.getId(), TableType.filter, ChainType.INPUT,1);
        testShowIptablesDetail("删除Input 1");
        iptablesUtils.delIptable(dockerContainer.getId(), ChainType.INPUT, "172.0.17.3", "", "DROP");
        testShowIptablesDetail("删除Input的条目");
        iptablesUtils.modifyIptable(dockerContainer.getId(), ChainType.OUTPUT, "172.0.17.2", "", "ACCEPT", 1);
        testShowIptablesDetail("modify output 1");
        iptablesUtils.flushIptables(dockerContainer.getId(), ChainType.OUTPUT);
        testShowIptablesDetail("flush output");

    }

    public void testShowIptablesDetail(String info) {
        System.out.println("---------------" + info +
                "--------------------");
        Map<String, List<ChainEntity>> iptablesList = iptablesUtils.showIptablesDetail(dockerContainer.getId());
        iptablesList.forEach((k, v) -> {
            System.out.println(k);
            v.forEach(s -> {
                System.out.println(s);
            });
        });
    }

    @After
    public void close() {
        dockerService.deleteDockerById(dockerContainer.getId());
    }
}
